#ifndef SCENEOBJECTS_HPP_INCLUDED
#define SCENEOBJECTS_HPP_INCLUDED

#include "../../Core/Classes/GameObject.hpp"
#include "../../Core/Classes/DecorativeObject.hpp"
#include "../../Core/Classes/GroundObject.hpp"
#include "../../Core/Classes/WallObject.hpp"
#include "../../Core/Classes/ObjectInfo.hpp"
#include "../../Core/Types/Tile.hpp"
#include "../../Core/Types/Convex.hpp"
#include <functional>
#include <cstdint>
#include <vector>
#include <string>

/** @addtogroup Models
* @{ */
class SceneObjects
{
    public:

        typedef enum OBJECT_TYPE
        {
            GAME_OBJECT         = (1 << 1),
            DECORATIVE_OBJECT   = (1 << 2),
            GROUND_OBJECT       = (1 << 3),
            WALL_OBJECT         = (1 << 4),
            ALL = (GAME_OBJECT | DECORATIVE_OBJECT | GROUND_OBJECT | WALL_OBJECT)
        } OBJECT_TYPE;

        static std::vector<Object> GetAll(OBJECT_TYPE ObjectTypes = ALL);
        static std::vector<Object> GetAll(const Tile& T, OBJECT_TYPE ObjectTypes = ALL);
        static std::vector<Object> GetAll(std::int32_t ID, OBJECT_TYPE ObjectTypes = ALL);
        static std::vector<Object> GetAll(const std::string& Name, OBJECT_TYPE ObjectTypes = ALL);
        static std::vector<Object> GetAll(const std::vector<std::int32_t>& PossibleIDs, OBJECT_TYPE ObjectTypes = ALL);
        static std::vector<Object> GetAll(const std::vector<std::string>& PossibleNames, OBJECT_TYPE ObjectTypes = ALL);
        static std::vector<Object> GetAll(const std::function<bool (const Object&)>& Filter, OBJECT_TYPE ObjectTypes = ALL);

        static Object Get(const Tile& T, OBJECT_TYPE ObjectTypes = ALL);
        static Object Get(std::int32_t ID, OBJECT_TYPE ObjectTypes = ALL);
        static Object Get(const std::string& Name, OBJECT_TYPE ObjectTypes = ALL);
        static Object Get(const std::vector<std::int32_t>& PossibleIDs, OBJECT_TYPE ObjectTypes = ALL);
        static Object Get(const std::vector<std::string>& PossibleNames, OBJECT_TYPE ObjectTypes = ALL);
        static Object Get(const std::function<bool (const Object&)>& Filter, OBJECT_TYPE ObjectTypes = ALL);

        static Internal::ObjectInfo GetInfoOf(const Object& O);
        static Internal::ObjectInfo GetInfoOf(const Internal::GameObject& G);
        static Internal::ObjectInfo GetInfoOf(const Internal::DecorativeObject& D);
        static Internal::ObjectInfo GetInfoOf(const Internal::GroundObject& G);
        static Internal::ObjectInfo GetInfoOf(const Internal::WallObject& W);

        static Tile GetTileOf(const Object& O);
        static Tile GetTileOf(const Internal::GameObject& G);
        static Tile GetTileOf(const Internal::DecorativeObject& D);
        static Tile GetTileOf(const Internal::GroundObject& G);
        static Tile GetTileOf(const Internal::WallObject& W);

        static Convex GetConvexOf(const Object& O);
        static Convex GetConvexOf(const Internal::GameObject& G);
        static Convex GetConvexOf(const Internal::DecorativeObject& D);
        static Convex GetConvexOf(const Internal::GroundObject& G);
        static Convex GetConvexOf(const Internal::WallObject& W);
};

/** @} */

#endif // SCENEOBJECTS_HPP_INCLUDED
